@@ -7,7 +7,6 @@ package Data::ObjectDriver::ResultSet;
 use strict;
 
 use base qw( Class::Accessor::Fast );
-use List::Util qw(min);
 
 ## Public/_Private Accessors
 
@@ -60,8 +59,6 @@ sub iterator {
 sub clone {
     my $self = shift;
 
-    # note that this is a SHALLOW copy--any values that are references
-    # will be shared between the original and the clone
     my $terms = $self->_terms ? { %{$self->_terms} } : {};
     my $args  = $self->_args  ? { %{$self->_args} }  : {};
 
@@ -94,7 +91,6 @@ sub add_constraint {
         my $cur_terms    = $self->_terms || {};
         my $filter_terms = $self->_filter_terms || {};
         foreach my $k (keys %$terms) {
-            $self->_results_loaded(0) unless $cur_terms->{$k};
             $cur_terms->{$k} = $terms->{$k};
             $filter_terms->{$k} = 1 if $self->_results_loaded;
         }
@@ -111,12 +107,10 @@ sub add_constraint {
         foreach my $k (keys %$args) {
             my $val = $args->{$k};
 
-            # If we get a limit arg that is bigger than our existing limit (and
-            # we *have* an existing limit), then
+            # If we get a limit arg that is bigger than our existing limit, then
             # make sure we force a requery.  Same for any filter arguments.
             # Same for offset arg that is  smaller than existing one.
-            if ((($k eq 'limit')  and
-            ( exists $cur_args->{'limit'} && defined $cur_args->{'limit'} && ($cur_args->{'limit'}||0)  < $val)) or
+            if ((($k eq 'limit')  and (($cur_args->{'limit'}||0)  < $val)) or
                 (($k eq 'offset') and (($cur_args->{'offset'}||0) > $val)) or
                 ($k eq 'filters')) {
                 $self->_results_loaded(0);
@@ -142,9 +136,7 @@ sub clear_constraint {
           if ref $term_names ne 'ARRAY';
 
         foreach my $n (@$term_names) {
-            if (delete $terms->{$n}) {
-                $self->_results_loaded(0);
-            }
+            delete $terms->{$n};
         }
     }
 
@@ -219,23 +211,6 @@ sub next {
     }
 }
 
-# look at next() without incrementing the cursor
-# like if you just want to see what's coming down the road at you
-sub peek_next {
-    my $self = shift;
-
-    return if $self->is_finished;
-
-    # Load the results and return an object
-    my $results = $self->_load_results;
-
-    my $obj = $results->[$self->_cursor + 1];
-
-    return $obj;
-}
-
-
-
 sub prev {
     my $self = shift;
 
@@ -264,14 +239,13 @@ sub curr {
 sub slice {
     my $self = shift;
     my ($start, $end) = @_;
+    my $limit = $end - $start;
 
     # Do we already have results?
     if ($self->_results) {
-        return [ @{ $self->_results }[$start..min($self->count-1, $end)] ];
+        return @{ $self->_results }[$start, $end];
     }
 
-    my $limit = $end - $start + 1;
-
     $self->add_offset($start);
     $self->add_limit($limit);
 
@@ -280,25 +254,10 @@ sub slice {
     return $r;
 }
 
-sub all {
-    my $self = shift;
-
-    return unless $self->count;
-
-    my @obj;
-    push @obj, $self->first;
-    while (my $obj = $self->next) {
-        push @obj, $obj;
-    }
-
-    $self->rewind;
-    return @obj;
-}
-
 sub count {
     my $self = shift;
 
-    # Get/load the results if we already have them
+    # Get/load the results if we already have them or if we have a limit term
     if ($self->_results_loaded or $self->get_limit) {
         my $results = $self->_load_results;
         return scalar @$results;
@@ -456,12 +415,6 @@ sub _load_results {
     return \@r;
 }
 
-sub rewind {
-    my $self = shift;
-    $self->is_finished(0);
-    $self->_cursor(-1);
-    return $self;
-}
 1;
 
 __END__
@@ -483,7 +436,7 @@ Data::ObjectDriver::ResultSet - Manage a DB query
     $result->add_limit(10);
     $result->add_offset(100);
 
-    while (not $result->is_finished) {
+    while (not $result->is_empty) {
         my $widget = $result->next;
 
         # Do stuff with $widget
@@ -786,39 +739,6 @@ Arguments:
 
   $obj = $res->next;
 
-=head2 peek_next
-
-Retrieve the next item in the resultset WITHOUT advancing the cursor.
-
-Arguments:
-
-=over 4
-
-=item I<none>
-
-=back
-
-; Return value
-: The next object or undef if past the end of the result set
-
-; Notes
-: Calling this method will force a DB query.  All subsequent calls to I<curr> will return this object
-
-; Example
-
-  while ($bottle = $res->next){
-
-      if ($bottle->type eq 'Bud Light'
-          && $res->peek_next->type eq 'Chimay'){
-
-          $bottle->pass; #don't spoil my palate
-
-      }else{
-          $bottle->drink;
-      }
-  }
-
-
 =head2 prev
 
 Retrieve the previous item in the result set
@@ -865,7 +785,7 @@ Arguments:
 
 =head2 slice
 
-Return a slice of the result set.  This is logically equivalent to setting a limit and offset and then retrieving all the objects via I<->next>.  If you call I<slice> and then call I<next>, you will get I<undef> and additionally I<is_finished> will be true.
+Return a slice of the result set.  This is logically equivalent to setting a limit and offset and then retrieving all the objects via I<->next>.  If you call I<slice> and then call I<next>, you will get I<undef> and additionally I<is_empty> will be true.
 
 Arguments:
 
@@ -936,13 +856,9 @@ Arguments:
 
 =head2 dod_debug
 
-Set this and you'll see $Data::ObjectDriver::DEBUG output when
+Set this and you'll see $Data::ObjectDriver::DEBUG output when 
 I go to get the results.
 
-=head2 rewind
-
-Move back to the start of the iterator for this instance of results of a query.
-
 =head2 first
 
 Returns the first object in the result set.
